// config.c

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#include "config.h"

static char *cfg_getStr(FILE *file)
{
	static char		line[4096];
	char			*str, *tmp;

	for(;;) {
		if(feof(file))
			return NULL;
		fgets(line, 4095, file);
		if(feof(file))
			return NULL;
		str = line;
		while(isspace(*str))			// Skip over spaces
			str++;
		if(strlen(str) == 0)			// The line is nothing but whitespace
			return NULL;
		if(str[0] != '#')				// The line is a comment
			break;
	}
	// Take whitespace off the back
	
	tmp = str + strlen(str) - 1;
	while(isspace(*tmp) && tmp != str)
		*(tmp--) = '\0';
	
	return str;
}

static BOOL cfg_readAllowIP(Config *cfg, FILE *file)
{
	char	*str;

	while((str = cfg_getStr(file)) != NULL) {
		AllowIP			*aip;
		unsigned char	a, b, c, d;
		unsigned int	mask;
		char			*tok;
		
		tok = strtok(str, ".");
		if(tok == NULL) goto error_ip;
		a = atoi(tok);
		if(a > 255) goto error_ip;
		tok = strtok(NULL, ".");
		if(tok == NULL) goto error_ip;
		b = atoi(tok);
		if(b > 255) goto error_ip;
		tok = strtok(NULL, ".");
		if(tok == NULL) goto error_ip;
		c = atoi(tok);
		if(c > 255) goto error_ip;
		tok = strtok(NULL, "/");
		if(tok == NULL) goto error_ip;
		d = atoi(tok);
		if(d > 255) goto error_ip;
		tok = strtok(NULL, "/");
		if(tok == NULL) goto error_mask;
		mask = atoi(tok);
		if(mask > 32) goto error_mask;

		cfg->aip = realloc(cfg->aip, (cfg->noAIP + 1) * sizeof(AllowIP));
		aip = &cfg->aip[cfg->noAIP++];
		aip->mask = (0xFFFFFFFF) << (32 - mask);
		aip->ip = ((a << 24) | (b << 16) | (c << 8) | (d)) & aip->mask;

#ifdef _DEBUG
		fprintf(stderr, "AIP %d: IP = %d.%d.%d.%d (%08x) - Mask = %08x\n", cfg->noAIP, a, b, c, d, aip->ip, aip->mask);
#endif
		
	}
	return TRUE;

error_mask:
	fprintf(stderr, "Invalid or missing mask number in the config file.\n");
	return FALSE;
error_ip:
	fprintf(stderr, "Malformed IP number in the config file.\n");
	return FALSE;
}

static BOOL cfg_readPathSub(Config *cfg, FILE *file)
{
	char	*str, *tmp;

	while((str = cfg_getStr(file)) != NULL) {
		char	split[2], *from, *to;
		// Split the string into two pieces
		
		tmp = strdup(str);
		split[0] = str[0];
		split[1] = '\0';
		str++;
		from = strtok(str, split);
		if(from == NULL)
			goto error_sub;
		to = strtok(NULL, split);
		if(to == NULL)
			goto error_sub;
		free(tmp);
		cfg->trfrom = realloc(cfg->trfrom, (cfg->noTrans + 1) * sizeof(LPSTR));
		cfg->trfrom[cfg->noTrans] = strdup(from);
		cfg->trto = realloc(cfg->trto, (cfg->noTrans + 1) * sizeof(LPSTR));
		cfg->trto[cfg->noTrans] = strdup(to);
		cfg->noTrans++;

#ifdef _DEBUG
		fprintf(stderr, "PS %d: From = '%s' - To = '%s'\n", cfg->noTrans, from, to);
#endif
	}
	return TRUE;

error_sub:
	fprintf(stderr, "Path substitution string '%s' is malformed.\n", tmp);
	free(tmp);
	return FALSE;
}

BOOL CFG_read(Config *cfg)
{
	FILE		*file;
	char		path[4096], *last;

	ZeroMemory(cfg, sizeof(*cfg));
	
	SearchPath(NULL, "rshd.exe", NULL, 4096, path, &last);
	sprintf(last, "rshd.cfg");

	if((file = fopen(path, "r")) == NULL) {
		fprintf(stderr, "Couldn't open configuration file (%s)...\n", path);
		return FALSE;
	}
	
	while(!feof(file)) {
		char	*str;
		
		str = cfg_getStr(file);
		if(!str)
			continue;
		
		if(stricmp(str, "[Allow IP]") == 0) {
			if(!cfg_readAllowIP(cfg, file))
				goto error;
			continue;
		}
		if(stricmp(str, "[Path Subst]") == 0) {
			if(!cfg_readPathSub(cfg, file))
				goto error;
			continue;
		}
	}
	fclose(file);
	return TRUE;

error:
	fclose(file);
	return FALSE;
}
